@@ -21,6 +21,7 @@ from bup.io import path_msg
 from bup.helpers import (Sha1, add_error, chunkyreader, debug1, debug2,
                          exo,
                          fdatasync,
+                         fsync,
                          finalized,
                          log,
                          merge_dict,
@@ -962,7 +963,7 @@ class PackWriter(object):
                                           b'objects/pack/pack-' +  hexlify(packbin))
                 os.rename(tmpdir + b'/pack', nameprefix + b'.pack')
                 os.rename(tmpdir + b'/idx', nameprefix + b'.idx')
-                os.fsync(pfd)
+                fsync(pfd)
                 if self.on_pack_finish:
                     self.on_pack_finish(nameprefix)
                 if run_midx:
@@ -1377,10 +1378,9 @@ class CatPipe:
             self.restart()
         assert(self.p)
         poll_result = self.p.poll()
-        assert(poll_result == None)
-        if self.inprogress:
-            log('get: opening %r while %r is open\n' % (ref, self.inprogress))
-        assert(not self.inprogress)
+        assert poll_result is None
+        assert not self.inprogress, \
+            f'opening {ref.decode("ascii")} while {self.inprogress.decode("ascii")} is open'
         assert ref.find(b'\n') < 0
         assert ref.find(b'\r') < 0
         assert not ref.startswith(b'-')
@@ -1408,15 +1408,14 @@ class CatPipe:
         if len(info) != 3 or len(info[0]) != 40:
             raise GitError('expected object (id, type, size), got %r' % info)
         oidx, typ, size = info
+        size = int(size)
 
         if not include_data:
             self.inprogress = None
             yield oidx, typ, size
             return
 
-        size = int(size)
         try:
-            it = chunkyreader(p.stdout, size)
             yield oidx, typ, size
             for blob in chunkyreader(p.stdout, size):
                 yield blob
@@ -1524,7 +1523,8 @@ def walk_object(get_ref, oidx, *, stop_at=None, include_data=None,
     include_data is logically false.  Missing blobs may not be noticed
     unless include_data is logically true or oid_exists(oid) is
     provided.  Yield items depth first, post-order, i.e. parents after
-    children.
+    children.  A tree will be yielded (later) if stop_item(oidx) is
+    false when it is first encountered.
 
     """
 
