@@ -1,6 +1,5 @@
 """Helper functions and classes for bup."""
 
-from __future__ import absolute_import, division
 from collections import namedtuple
 from contextlib import ExitStack
 from ctypes import sizeof, c_void_p
@@ -8,6 +7,7 @@ from math import floor
 from os import environ
 from subprocess import PIPE, Popen
 from tempfile import mkdtemp
+from traceback import print_exception
 from shutil import rmtree
 import sys, os, subprocess, errno, select, mmap, stat, re, struct
 import hashlib, heapq, math, operator, time
@@ -21,6 +21,17 @@ from bup.io import byte_stream, path_msg
 from bup.options import _tty_width as tty_width
 
 
+# EXIT_TRUE (just an alias) and EXIT_FALSE are intended for cases like
+# POSIX grep or test, or bup's own "fsck --par2-ok", where the command
+# is asking a question with a yes or no answer.  Eventually all
+# commands should avoid exiting with 1 for errors.
+
+EXIT_SUCCESS = 0
+EXIT_TRUE = 0
+EXIT_FALSE = 1
+EXIT_FAILURE = 2
+
+
 buglvl = int(os.environ.get('BUP_DEBUG', 0))
 
 
@@ -33,11 +44,28 @@ def nullcontext_if_not(manager):
     return manager if manager is not None else nullcontext()
 
 
+def getgroups():
+    # cf. getgroups(2) - effective group id may or may not be in the
+    # list, and while on linux, for example, it normally is, in an
+    # unshare, it wasn't.
+    egid = os.getegid()
+    gids = os.getgroups()
+    if egid not in gids:
+        gids.append(egid)
+    return gids
+
+
 class finalized:
-    def __init__(self, enter_result=None, finalize=None):
-        assert finalize
-        self.finalize = finalize
-        self.enter_result = enter_result
+    def __init__(self, what_or_how, how=None):
+        # pyupgrade 3.8+: add positional-only '/' to arguments
+        # Finalizer is required, value is not, parameters are only
+        # positional.
+        if how is None:
+            self.enter_result = None
+            self.finalize = what_or_how
+        else:
+            self.enter_result = what_or_how
+            self.finalize = how
     def __enter__(self):
         return self.enter_result
     def __exit__(self, exc_type, exc_value, traceback):
@@ -73,7 +101,7 @@ except AttributeError:
 if sys.platform.startswith('darwin'):
     # Apparently os.fsync on OS X doesn't guarantee to sync all the way down
     import fcntl
-    def fdatasync(fd):
+    def fsync(fd):
         try:
             return fcntl.fcntl(fd, fcntl.F_FULLFSYNC)
         except IOError as e:
@@ -82,8 +110,10 @@ if sys.platform.startswith('darwin'):
                 return _fdatasync(fd)
             else:
                 raise
+    fdatasync = fsync
 else:
-    fdatasync = _fdatasync
+    fdatasync = getattr(os, 'fdatasync', os.fsync) # currently always fdatasync
+    fsync = os.fsync
 
 
 def partition(predicate, stream):
@@ -413,7 +443,7 @@ def detect_fakeroot():
 if sys.platform.startswith('cygwin'):
     def is_superuser():
         # https://cygwin.com/ml/cygwin/2015-02/msg00057.html
-        groups = os.getgroups()
+        groups = getgroups()
         return 544 in groups or 0 in groups
 else:
     def is_superuser():
@@ -770,6 +800,9 @@ class atomically_replaced_file:
         with self.cleanup:
             if not (self.canceled or exc_type):
                 os.rename(self.tmp_path, self.path)
+                parent = os.path.dirname(self.path) or b'.'
+                with finalized(os.open(parent, os.O_RDONLY), os.close) as fd:
+                    fsync(fd)
     def cancel(self):
         self.canceled = True
 
@@ -784,18 +817,18 @@ def slashappend(s):
 
 
 def _mmap_do(f, sz, flags, prot, close):
-    if not sz:
-        st = os.fstat(f.fileno())
-        sz = st.st_size
-    if not sz:
-        # trying to open a zero-length map gives an error, but an empty
-        # string has all the same behaviour of a zero-length map, ie. it has
-        # no elements :)
-        return ''
-    map = io.mmap(f.fileno(), sz, flags, prot)
-    if close:
-        f.close()  # map will persist beyond file close
-    return map
+    with ExitStack() as contexts:
+        if close:
+            contexts.enter_context(f)
+        if not sz:
+            st = os.fstat(f.fileno())
+            sz = st.st_size
+        if not sz:
+            # trying to open a zero-length map gives an error, but an empty
+            # string has all the same behaviour of a zero-length map, ie. it has
+            # no elements :)
+            return b''
+        return io.mmap(f.fileno(), sz, flags, prot)
 
 
 def mmap_read(f, sz = 0, close=True):
@@ -933,6 +966,10 @@ def add_error(e):
     saved_errors.append(e)
     log('%-70s\n' % e)
 
+def note_error(m):
+    # FIXME: rework console output, logging, and api...
+    saved_errors.append(m)
+    log(m)
 
 def clear_errors():
     global saved_errors
@@ -1192,7 +1229,7 @@ if _localtime:
 else:
     localtime = time.localtime
     def utc_offset_str(t):
-        return time.strftime(b'%z', localtime(t))
+        return time.strftime('%z', localtime(t)).encode('ascii')
     def to_py_time(x):
         return x
 
@@ -1233,3 +1270,23 @@ def period_as_secs(s):
                   b'w': 60 * 60 * 24 * 7,
                   b'm': 60 * 60 * 24 * 31,
                   b'y': 60 * 60 * 24 * 366}[scale]
+
+
+def wrap_boolean_main(main):
+    # FIXME: this will eventually migrate to main.py when we clean up
+    # exit statuses.  It's here for now because we need to make sure
+    # we only exit EXIT_TRUE for main() EXIT_TRUE, since other code
+    # may not respect the distinction, and because python's default
+    # error exit status is 1.  See also compat.wrap_main().
+    try:
+        rc = main()
+    except KeyboardInterrupt: # preserve existing behavior for now
+        sys.exit(130)
+    except SystemExit as ex:
+        sys.exit(EXIT_FAILURE if ex.code == EXIT_TRUE else ex.code)
+    except BaseException as ex:
+        print_exception(ex)
+        sys.exit(EXIT_FAILURE)
+    if rc:
+        sys.exit(rc)
+    die_if_errors(status=EXIT_FAILURE)
