@@ -37,13 +37,12 @@ class PackMidx:
     """
     def __init__(self, filename, mmap, *, _internal=False):
         """Takes ownership of mmap."""
-        with ExitStack() as contexts:
-            contexts.enter_context(mmap)
+        self.closed = finished = False
+        try:
+            self.map = mmap
             assert _internal, 'call open_midx()'
             assert _midx_header(mmap) == MIDX_HEADER
             assert _midx_version(mmap) == MIDX_VERSION
-            self.map = mmap
-            self.closed = False
             self.name = filename
             self.bits = _helpers.firstword(self.map[8:12])
             self.entries = 2**self.bits
@@ -55,15 +54,26 @@ class PackMidx:
             self.which_ofs = self.sha_ofs + 20 * self.nsha
             # which len is self.nsha * 4
             self.idxnames = self.map[self.which_ofs + 4 * self.nsha:].split(b'\0')
-            # REVIEW: idx paths always relative to midx path?
-            idxdir = os.path.dirname(filename) + b'/'
+            idxdir = os.path.dirname(filename)
             missing = []
             for name in self.idxnames:
-                if not os.path.exists(idxdir + name):
-                    self.missing.append(name)
+                if not os.path.exists(os.path.join(idxdir, name)):
+                    missing.append(name)
             if missing:
                 raise MissingIdxs(paths=missing)
-            contexts.pop_all()
+            finished = True
+        finally:
+            if not finished: self.close()
+
+    def close(self):
+        # This must be able to handle __init__ partial initializations too.
+        if not self.closed:
+            self.closed = True
+            self.fanout = self.shatable = self.whichlist = self.idxnames = None
+            tmp = getattr(self, 'map', None)
+            if tmp is not None:
+                self.map = None
+                tmp.close()
 
     def __enter__(self):
         return self
@@ -93,13 +103,6 @@ class PackMidx:
     def _get_idxname(self, i):
         return self.idxnames[self._get_idx_i(i)]
 
-    def close(self):
-        self.closed = True
-        if self.map is not None:
-            self.fanout = self.shatable = self.whichlist = self.idxnames = None
-            self.map.close()
-            self.map = None
-
     def __del__(self):
         assert self.closed
 
@@ -176,9 +179,9 @@ def open_midx(path, *, ignore_missing=True):
             if not missing:
                 return midx
             pathm = path_msg(path)
-            for missing in ex.paths:
-                imsg = path_msg(missing)
-                log(f'Warning: ignoring midx {pathm} (missing idx {imsg})\n')
+            for idx in missing:
+                idxm = path_msg(idx)
+                log(f'Warning: ignoring midx {pathm} (missing idx {idxm})\n')
             return None
         pathm = path_msg(path)
         if ver < MIDX_VERSION:
